/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.camel.component.servlet.rest;

import org.apache.camel.BindToRegistry;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.component.servlet.ServletCamelRouterTestSupport;
import org.apache.camel.component.servlet.ServletRestHttpBinding;
import org.apache.camel.model.rest.RestParamType;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class RestServletQueryParamTest extends ServletCamelRouterTestSupport {

    @BindToRegistry("myBinding")
    private ServletRestHttpBinding restHttpBinding = new ServletRestHttpBinding();

    @Test
    public void testQueryTrue() throws Exception {
        WebRequest req = new GetMethodWebRequest(contextUrl + "/services/users/");
        req.setParameter("auth", "secret");
        WebResponse response = query(req, false);

        assertEquals(200, response.getResponseCode());

        assertEquals("secret;Donald Duck", response.getText());
    }

    @Test
    public void testQueryFalse() throws Exception {
        WebRequest req = new GetMethodWebRequest(contextUrl + "/services/users/");
        WebResponse response = query(req, false);

        assertEquals(400, response.getResponseCode());
    }

    @Override
    protected RouteBuilder createRouteBuilder() throws Exception {
        return new RouteBuilder() {
            @Override
            public void configure() throws Exception {
                // configure to use servlet on localhost
                restConfiguration().component("servlet").host("localhost").endpointProperty("httpBinding", "#myBinding")
                        .clientRequestValidation(true);

                // use the rest DSL to define the rest services
                rest()
                    .get("/users/")
                        .param()
                            .name("auth")
                            .type(RestParamType.query)
                            .required(true)
                        .endParam()
                    .route().to("mock:input").process(exchange -> {
                        String auth = exchange.getIn().getHeader("auth", String.class);
                            exchange.getMessage().setBody(auth + ";Donald Duck");
                        });
            }
        };
    }

}
